﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GridView;

namespace AddNewRowToGenericListSample
{
	public class UndoHelper<TDataItem> where TDataItem : class
	{
		RadGridView gridView;

		public UndoHelper(RadGridView gridView)
		{
			this.gridView = gridView;
			items = (IList<TDataItem>) this.gridView.ItemsSource;
		}

		private TDataItem newlyAddedDataItem;
		private IList<TDataItem> items;

		public void DataItemCreated(TDataItem item)
		{
			this.newlyAddedDataItem = item;
		}

		public void UndoEdit(GridViewCell cell, object oldCellValue)
		{
			var editedDataItem = (TDataItem) cell.DataContext;

			var editedProperty = TypeDescriptor.GetProperties(editedDataItem)[cell.DataColumn.DataMemberPath];
			editedProperty.SetValue(editedDataItem, oldCellValue);

			if (object.ReferenceEquals(editedDataItem, this.newlyAddedDataItem))
			{
				var itemToDelete = newlyAddedDataItem;
				this.gridView.Dispatcher.BeginInvoke((Action) delegate()
                  	{
                  		this.items.Remove(itemToDelete);
                  		RebindGridView();
                  	});
			}
		}

		private void RebindGridView()
		{
			this.gridView.ItemsSource = null;
			this.gridView.ItemsSource = this.items;
		}

		public void EditEnded()
		{
			if (this.newlyAddedDataItem != null)
			{
				this.items.Add(newlyAddedDataItem);
			}

			this.newlyAddedDataItem = null;
		}
	}
}
