﻿using System;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections;
using System.Collections.Generic;
using Telerik.Windows.Controls.GridView;

namespace AddNewRowToGenericListSample
{
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>
	public partial class Window1 : Window
	{
		private UndoHelper<Person> undoHelper;

		public Window1()
		{
			InitializeComponent();
			this.RadGridView1.ItemsSource = GetPersons();
			this.RadGridView1.AddingNewDataItem += new EventHandler<GridViewAddingNewEventArgs>(RadGridView1_AddingNewDataItem);

			this.undoHelper = new UndoHelper<Person>(this.RadGridView1);

		}

		void RadGridView1_AddingNewDataItem(object sender, GridViewAddingNewEventArgs e)
		{
			var newlyAddedPerson = new Person { ID = 0, Name = "<type name here>" };
			this.undoHelper.DataItemCreated(newlyAddedPerson);
			e.NewObject = newlyAddedPerson;
		}

		private void RadGridView1_EditEnded(object sender, Telerik.Windows.Controls.GridView.Cells.CellRoutedEventArgs e)
		{
			var cell = (GridViewCell) e.OriginalSource;

			//determine if we should rollback the visual edit (DB error)
			var shouldRollBack = true;
			if (shouldRollBack)
			{
				this.undoHelper.UndoEdit(cell, e.OldValue);
			}

			this.undoHelper.EditEnded();
		}

		private static IEnumerable GetPersons()
		{
			var persons = new List<Person>();

			persons.Add(new Person { ID = 1, Name = "John" });
			persons.Add(new Person { ID = 2, Name = "Jane" });
			persons.Add(new Person { ID = 3, Name = "George" });

			return persons;
		}

		private void Button_Click(object sender, RoutedEventArgs e)
		{
			PressInsertKeyOnRadGridView();
		}

		private void PressInsertKeyOnRadGridView()
		{
			var itemsControl = this.RadGridView1.ItemsControl;
			itemsControl.Focus();

			// Simulate INSERT key is pressed from Framework
			var keyEventArgs = new KeyEventArgs(Keyboard.PrimaryDevice, Keyboard.PrimaryDevice.ActiveSource, 0, Key.Insert);
			keyEventArgs.RoutedEvent = UIElement.KeyDownEvent;
			InputManager.Current.ProcessInput(keyEventArgs);
		}

	}

	public class Person
	{
		public int ID { get; set; }
		public string Name { get; set; }
	}
}
